@@ -1,4 +1,4 @@
-/* 07jul14abu
+/* 15sep14abu
  * (c) Software Lab. Alexander Burger
  */
 
@@ -28,8 +28,10 @@ any doPort(any ex) {
       ipErr(ex, "socket");
    closeOnExec(ex, sd);
    n = 0;
+#ifndef __OpenBSD__
    if (setsockopt(sd, IPPROTO_IPV6, IPV6_V6ONLY, &n, sizeof(n)) < 0)
       ipErr(ex, "IPV6_V6ONLY");
+#endif
    memset(&addr, 0, sizeof(addr));
    addr.sin6_family = AF_INET6;
    addr.sin6_addr = in6addr_any;
