@@ -1,4 +1,4 @@
-/* 29oct13abu
+/* 19jun14abu
  * (c) Software Lab. Alexander Burger
  */
 
@@ -24,6 +24,9 @@ typedef enum {NO,YES} bool;
 
 static char *File, *Dir, *Data;
 static off_t Size;
+static bool Hot;
+
+static char Ciphers[] = "ECDHE-RSA-RC4-SHA:RC4:HIGH:!MD5:!aNULL:!EDH";
 
 static char Get[] =
    "GET /%s HTTP/1.0\r\n"
@@ -31,22 +34,11 @@ static char Get[] =
    "Host: %s:%s\r\n"
    "Accept-Charset: utf-8\r\n\r\n";
 
-static void errmsg(char *msg) {
-   fprintf(stderr, "ssl: %s\n", msg);
-}
-
 static void giveup(char *msg) {
-   errmsg(msg);
+   fprintf(stderr, "ssl: %s\n", msg);
    exit(1);
 }
 
-static void sslChk(int n) {
-   if (n < 0) {
-      ERR_print_errors_fp(stderr);
-      exit(1);
-   }
-}
-
 static int sslConnect(SSL *ssl, char *node, char *service) {
    struct addrinfo hints, *lst, *p;
    int sd;
@@ -94,19 +86,40 @@ static bool sslFile(SSL *ssl, char *file) {
    return n == 0;
 }
 
-static void doSigTerm(int n __attribute__((unused))) {
-   int fd1, fd2, cnt;
-   char buf[BUFSIZ];
+static void lockFile(int fd) {
+   struct flock fl;
 
-   if (Data  &&  (fd1 = open(File, O_RDWR)) >= 0) {
-      if (unlink(File) < 0)
-         giveup("Can't unlink back");
-      if ((fd2 = open(File, O_CREAT|O_WRONLY|O_TRUNC, 0666)) < 0)
-         giveup("Can't create back");
-      if (write(fd2, Data, Size) != Size)
-         giveup("Can't write back");
-      while ((cnt = read(fd1, buf, sizeof(buf))) > 0)
-         write(fd2, buf, cnt);
+   fl.l_type = F_WRLCK;
+   fl.l_whence = SEEK_SET;
+   fl.l_start = 0;
+   fl.l_len = 0;
+   if (fcntl(fd, F_SETLKW, &fl) < 0)
+      giveup("Can't lock");
+}
+
+static void doSigTerm(int n __attribute__((unused))) {
+   int fd;
+   struct stat st;
+   char *data;
+
+   if (Hot) {
+      if ((fd = open(File, O_RDWR)) < 0)
+         giveup("Can't final open");
+      lockFile(fd);
+      if (fstat(fd,&st) < 0)
+         giveup("Can't final access");
+      if (st.st_size != 0) {
+         if ((data = malloc(st.st_size)) == NULL)
+            giveup("Can't final alloc");
+         if (read(fd, data, st.st_size) != st.st_size)
+            giveup("Can't final read");
+         if (ftruncate(fd,0) < 0)
+            giveup("Can't final truncate");
+      }
+      if (write(fd, Data, Size) != Size)
+         giveup("Can't final write (1)");
+      if (st.st_size != 0  &&  write(fd, data, st.st_size) != st.st_size)
+         giveup("Can't final write (2)");
    }
    exit(0);
 }
@@ -116,15 +129,17 @@ static void doSigTerm(int n __attribute__((unused))) {
 // ssl host port url key file
 // ssl host port url key file dir sec
 int main(int ac, char *av[]) {
+   bool dbg;
    SSL_CTX *ctx;
    SSL *ssl;
    int n, sec, getLen, lenLen, fd, sd;
    DIR *dp;
    struct dirent *p;
    struct stat st;
-   struct flock fl;
    char get[1024], buf[4096], nm[4096], len[64];
 
+   if (dbg = strcmp(av[ac-1], "+") == 0)
+      --ac;
    if (!(ac >= 4 && ac <= 6  ||  ac == 8))
       giveup("host port url [[key] file] | host port url key file dir sec");
    if (strlen(Get)+strlen(av[1])+strlen(av[2])+strlen(av[3]) >= sizeof(get))
@@ -137,14 +152,18 @@ int main(int ac, char *av[]) {
       ERR_print_errors_fp(stderr);
       giveup("SSL init");
    }
+   SSL_CTX_set_cipher_list(ctx, Ciphers);
    ssl = SSL_new(ctx);
 
    if (ac <= 6) {
       if (sslConnect(ssl, av[1], av[2]) < 0) {
-         errmsg("Can't connect");
-         return 1;
+         ERR_print_errors_fp(stderr);
+         giveup("Can't connect");
+      }
+      if (SSL_write(ssl, get, getLen) < 0) {
+         ERR_print_errors_fp(stderr);
+         giveup("SSL GET");
       }
-      sslChk(SSL_write(ssl, get, getLen));
       if (ac > 4) {
          if (*av[4]  &&  !sslFile(ssl,av[4]))
             giveup(av[4]);
@@ -153,16 +172,18 @@ int main(int ac, char *av[]) {
       }
       while ((n = SSL_read(ssl, buf, sizeof(buf))) > 0)
          write(STDOUT_FILENO, buf, n);
+      if (dbg)
+         ERR_print_errors_fp(stderr);
       return 0;
    }
-
-   signal(SIGCHLD,SIG_IGN);  /* Prevent zombies */
-   if ((n = fork()) < 0)
-      giveup("detach");
-   if (n)
-      return 0;
-   setsid();
-
+   if (!dbg) {
+      signal(SIGCHLD,SIG_IGN);  /* Prevent zombies */
+      if ((n = fork()) < 0)
+         giveup("detach");
+      if (n)
+         return 0;
+      setsid();
+   }
    File = av[5];
    Dir = av[6];
    sec = atoi(av[7]);
@@ -175,12 +196,7 @@ int main(int ac, char *av[]) {
          if (fstat(fd,&st) < 0  ||  st.st_size == 0)
             close(fd);
          else {
-            fl.l_type = F_WRLCK;
-            fl.l_whence = SEEK_SET;
-            fl.l_start = 0;
-            fl.l_len = 0;
-            if (fcntl(fd, F_SETLKW, &fl) < 0)
-               giveup("Can't lock");
+            lockFile(fd);
             if (fstat(fd,&st) < 0  ||  (Size = st.st_size) == 0)
                giveup("Can't access");
             lenLen = sprintf(len, "%ld\n", Size);
@@ -188,8 +204,9 @@ int main(int ac, char *av[]) {
                giveup("Can't alloc");
             if (read(fd, Data, Size) != Size)
                giveup("Can't read");
+            Hot = YES;
             if (ftruncate(fd,0) < 0)
-               errmsg("Can't truncate");
+               giveup("Can't truncate");
             close(fd);
             for (;;) {
                if ((sd = sslConnect(ssl, av[1], av[2])) >= 0) {
@@ -200,6 +217,7 @@ int main(int ac, char *av[]) {
                            SSL_write(ssl, Data, Size) == Size  &&    // data
                            SSL_write(ssl, "T", 1) == 1  &&           // ack
                            SSL_read(ssl, buf, 1) == 1  &&  buf[0] == 'T' ) {
+                     Hot = NO;
                      alarm(0);
                      sslClose(ssl,sd);
                      break;
@@ -207,9 +225,11 @@ int main(int ac, char *av[]) {
                   alarm(0);
                   sslClose(ssl,sd);
                }
+               if (dbg)
+                  ERR_print_errors_fp(stderr);
                sleep(sec);
             }
-            free(Data),  Data = NULL;
+            free(Data);
          }
       }
       if (*Dir && (dp = opendir(Dir))) {
@@ -226,6 +246,8 @@ int main(int ac, char *av[]) {
                      unlink(nm);
                   sslClose(ssl,sd);
                }
+               if (dbg)
+                  ERR_print_errors_fp(stderr);
             }
          }
          closedir(dp);
